/*
 * FileDisk Shell Extension
 * Copyright (c) 2005-2007 Gianluigi Tiesi <sherpya@netfarm.it>
 * ShellExt Code taken from ClamWin Antivirus Scanner (c) 2004 alch,
 * FileDisk Code taken from FileDisk (c) 1999-2004 Bo Brantn.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "FileDiskExt.h"

int CShellExt::FileDiskMount(int DeviceNumber, POPEN_FILE_INFORMATION  OpenFileInformation, TCHAR DriveLetter, BOOLEAN CdImage)
{
    TCHAR   VolumeName[] = _T("\\\\.\\ :");
    TCHAR   DeviceName[255];
    HANDLE  Device = INVALID_HANDLE_VALUE;
    DWORD   BytesReturned;
    DWORD   res = -1;

    VolumeName[4] = DriveLetter;

    Device = CreateFile(
        VolumeName,
        GENERIC_READ | GENERIC_WRITE,
        FILE_SHARE_READ | FILE_SHARE_WRITE,
        NULL,
        OPEN_EXISTING,
        FILE_FLAG_NO_BUFFERING,
        NULL
        );

    if (Device != INVALID_HANDLE_VALUE)
    {
        SetLastError(ERROR_BUSY);
        goto end_mount;
    }

    if (CdImage)
        _stprintf(DeviceName, DEVICE_NAME_PREFIX _T("Cd") _T("%u"), DeviceNumber);
    else
        _stprintf(DeviceName, DEVICE_NAME_PREFIX _T("%u"), DeviceNumber);

    if (!DefineDosDevice(
        DDD_RAW_TARGET_PATH,
        &VolumeName[4],
        DeviceName
        ))
        goto end_mount;

    Device = CreateFile(
        VolumeName,
        GENERIC_READ | GENERIC_WRITE,
        FILE_SHARE_READ | FILE_SHARE_WRITE,
        NULL,
        OPEN_EXISTING,
        FILE_FLAG_NO_BUFFERING,
        NULL
        );

    if (Device == INVALID_HANDLE_VALUE)
    {
        DefineDosDevice(DDD_REMOVE_DEFINITION, &VolumeName[4], NULL);
        goto end_mount;
    }

    if (!DeviceIoControl(
        Device,
        IOCTL_FILE_DISK_OPEN_FILE,
        OpenFileInformation,
        sizeof(OPEN_FILE_INFORMATION),
        NULL,
        0,
        &BytesReturned,
        NULL
        ))
    {
        DefineDosDevice(DDD_REMOVE_DEFINITION, &VolumeName[4], NULL);
        goto end_mount;
    }

    res = 0;
end_mount:
    if (Device != INVALID_HANDLE_VALUE) CloseHandle(Device);
    return res;
}

int CShellExt::FileDiskStatus(TCHAR DriveLetter)
{
    TCHAR                   VolumeName[] = _T("\\\\.\\ :");
    HANDLE                  Device;
    OPEN_FILE_INFORMATION   OpenFileInformation;
    DWORD                   BytesReturned;
    int res = -1;
    DWORD len = 0;

    VolumeName[4] = DriveLetter;

    Device = CreateFile(VolumeName,
        GENERIC_READ,
        FILE_SHARE_READ | FILE_SHARE_WRITE,
        NULL,
        OPEN_EXISTING,
        FILE_FLAG_NO_BUFFERING,
        NULL
        );

    if (Device == INVALID_HANDLE_VALUE)
        return res;

    memset(&OpenFileInformation, 0, sizeof(OpenFileInformation));

    if (!DeviceIoControl(
        Device,
        IOCTL_FILE_DISK_QUERY_FILE,
        NULL,
        0,
        &OpenFileInformation,
        sizeof(OPEN_FILE_INFORMATION),
        &BytesReturned,
        NULL
        ))
        goto end_status;

    if (BytesReturned < sizeof(OPEN_FILE_INFORMATION) - MAX_PATH)
        goto end_status;

#ifdef _UNICODE
    if (!MultiByteToWideChar(CP_ACP, MB_ERR_INVALID_CHARS, (LPCSTR) &OpenFileInformation.FileName[4], OpenFileInformation.FileNameLength - 3, m_szMountedImage, MAX_PATH))
        return -1;
#else
    len = OpenFileInformation.FileNameLength - 3;
    memcpy(m_szMountedImage, &OpenFileInformation.FileName[4], len - 1);
    m_szMountedImage[len - 1] = 0;
#endif

    if (!_tcsncmp(m_szMountedImage, _T("UNC"), 3))
        _stprintf(m_szMountedImage, _T("\\%s"), &m_szMountedImage[3]);

    res = 0;

end_status:
    CloseHandle(Device);
    return res;
}


int CShellExt::FileDiskUmount(TCHAR DriveLetter)
{
    TCHAR    VolumeName[] = _T("\\\\.\\ :");
    HANDLE  Device;
    DWORD   BytesReturned;

    VolumeName[4] = DriveLetter;

    Device = CreateFile(
        VolumeName,
        GENERIC_READ | GENERIC_WRITE,
        FILE_SHARE_READ | FILE_SHARE_WRITE,
        NULL,
        OPEN_EXISTING,
        FILE_FLAG_NO_BUFFERING,
        NULL
        );

    if (Device == INVALID_HANDLE_VALUE)
        return -1;

    if (!DeviceIoControl(
        Device,
        FSCTL_LOCK_VOLUME,
        NULL,
        0,
        NULL,
        0,
        &BytesReturned,
        NULL
        ))
        goto end_umount;

    if (!DeviceIoControl(
        Device,
        IOCTL_FILE_DISK_CLOSE_FILE,
        NULL,
        0,
        NULL,
        0,
        &BytesReturned,
        NULL
        ))
        goto end_umount;

    if (!DeviceIoControl(
        Device,
        FSCTL_DISMOUNT_VOLUME,
        NULL,
        0,
        NULL,
        0,
        &BytesReturned,
        NULL
        ))
        goto end_umount;

    if (!DeviceIoControl(
        Device,
        FSCTL_UNLOCK_VOLUME,
        NULL,
        0,
        NULL,
        0,
        &BytesReturned,
        NULL
        ))
        goto end_umount;

    CloseHandle(Device);
    if (DefineDosDevice(DDD_REMOVE_DEFINITION, &VolumeName[4], NULL))
        return 0;
    else
        return -1;

end_umount:
    CloseHandle(Device);
    return -1;
}
