/*
 * FileDisk Shell Extension
 * Copyright (c) 2005-2007 Gianluigi Tiesi <sherpya@netfarm.it>
 * ShellExt Code taken from ClamWin Antivirus Scanner (c) 2004 alch,
 * FileDisk Code taken from FileDisk (c) 1999-2004 Bo Brantn.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "FileDiskExt.h"

UINT      g_cRefThisDll = 0;    /* Reference count of this DLL */
HINSTANCE g_hmodThisDll = NULL; /* Handle to this DLL itself */

BOOL APIENTRY DllMain(HINSTANCE hInstance, DWORD reason, LPVOID lpReserved)
{
    switch (reason)
    {
        case DLL_PROCESS_ATTACH:
            DisableThreadLibraryCalls(hInstance);
            g_hmodThisDll = hInstance;
        case DLL_THREAD_ATTACH:
        case DLL_THREAD_DETACH:
        case DLL_PROCESS_DETACH:
            break;
    }
    return TRUE;
}

/* DllCanUnloadNow */
STDAPI DllCanUnloadNow(void)
{
    return (g_cRefThisDll == 0 ? S_OK : S_FALSE);
}

/* DllGetClassObject */
STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID *ppvOut)
{
    *ppvOut = NULL;

    if (IsEqualIID(rclsid, CLSID_FileDiskExtension))
    {
        CShellExtClassFactory *pcf = new CShellExtClassFactory;
        return pcf->QueryInterface(riid, ppvOut);
    }

    return CLASS_E_CLASSNOTAVAILABLE;
}

/* CShellExtClassFactory::CShellExtClassFactory */
CShellExtClassFactory::CShellExtClassFactory()
{
    m_cRef = 0L;
    g_cRefThisDll++;
}

/* CShellExtClassFactory::~CShellExtClassFactory */
CShellExtClassFactory::~CShellExtClassFactory()
{
    g_cRefThisDll--;
}

/* CShellExtClassFactory::QueryInterface */
STDMETHODIMP CShellExtClassFactory::QueryInterface(REFIID riid, LPVOID FAR *ppv)
{
    *ppv = NULL;

    /* Any interface on this object is the object pointer */
    if (IsEqualIID(riid, IID_IUnknown) || IsEqualIID(riid, IID_IClassFactory))
    {
        *ppv = (LPCLASSFACTORY)this;
        AddRef();
        return NOERROR;
    }
    return E_NOINTERFACE;
}

/* CShellExtClassFactory::AddRef */
STDMETHODIMP_(ULONG) CShellExtClassFactory::AddRef()
{
    return ++m_cRef;
}

/* CShellExtClassFactory::Release */
STDMETHODIMP_(ULONG) CShellExtClassFactory::Release()
{
    if (--m_cRef)
        return m_cRef;

    delete this;
    return 0L;
}

/* CShellExtClassFactory::CreateInstance */
STDMETHODIMP CShellExtClassFactory::CreateInstance(LPUNKNOWN pUnkOuter, REFIID riid, LPVOID *ppvObj)
{
    *ppvObj = NULL;

    /* Shell extensions typically don't support aggregation (inheritance) */

    if (pUnkOuter)
        return CLASS_E_NOAGGREGATION;

    /*
        Create the main shell extension object.  The shell will then call
        QueryInterface with IID_IShellExtInit--this is how shell extensions are
        initialized.
    */

    LPCSHELLEXT pShellExt = new CShellExt();  /* Create the CShellExt object */

    if (NULL == pShellExt)
        return E_OUTOFMEMORY;
    return pShellExt->QueryInterface(riid, ppvObj);
}

/* CShellExtClassFactory::LockServer */
STDMETHODIMP CShellExtClassFactory::LockServer(BOOL fLock)
{
    return NOERROR;
}

/************************ CShellExt **************************/

/* CShellExt::CShellExt */
CShellExt::CShellExt()
{
    HKEY hKey;

    m_cRef = 0L;
    m_pDataObj = NULL;
    m_szPath[0] = 0;
    m_szMountedImage[0] = 0;

    m_MountedOn = -1;
    m_Type = TYPE_NOT_HANDLED;
    m_NumberOfDevices = 0;

    m_hFileDiskBmp = LoadBitmap(g_hmodThisDll, MAKEINTRESOURCE(IDB_FILEDISK));

    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, NUMBEROFDEVICES, 0, KEY_READ | KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS)
    {
        DWORD Len = sizeof(DWORD);
        DWORD Type = 0;

        RegQueryValueEx(hKey, _T("NumberOfDevices"), 0, &Type, (LPBYTE) &m_NumberOfDevices, &Len);
        RegCloseKey(hKey);
    }

    g_cRefThisDll++;
}

/* CShellExt::~CShellExt */
CShellExt::~CShellExt()
{
    if (m_pDataObj)
        m_pDataObj->Release();

    g_cRefThisDll--;
}

/* CShellExt::QueryInterface */
STDMETHODIMP CShellExt::QueryInterface(REFIID riid, LPVOID FAR *ppv)
{
    *ppv = NULL;

    if (IsEqualIID(riid, IID_IShellExtInit) || IsEqualIID(riid, IID_IUnknown))
        *ppv = (LPSHELLEXTINIT)this;
    else if (IsEqualIID(riid, IID_IContextMenu))
        *ppv = (LPCONTEXTMENU)this;

    if (*ppv)
    {
        AddRef();
        return NOERROR;
    }
    return E_NOINTERFACE;
}

/* CShellExt::AddRef */
STDMETHODIMP_(ULONG) CShellExt::AddRef()
{
    return ++m_cRef;
}

/* CShellExt::Release */
STDMETHODIMP_(ULONG) CShellExt::Release()
{
    if (--m_cRef)
        return m_cRef;

    delete this;
    return 0L;
}
