/*
 * FileDisk Shell Extension
 * Copyright (c) 2005-2007 Gianluigi Tiesi <sherpya@netfarm.it>
 * ShellExt Code taken from ClamWin Antivirus Scanner (c) 2004 alch,
 * FileDisk Code taken from FileDisk (c) 1999-2004 Bo Brantn.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#define WIN32_LEAN_AND_MEAN
#define _CRT_SECURE_NO_DEPRECATE
#define _CRT_NON_CONFORMING_SWPRINTFS

#include <windows.h>
#include <winioctl.h>
#include <tchar.h>
#include <stdio.h>
#include <shlobj.h>
#include <shlwapi.h>
#include <shellapi.h>
#include <shlguid.h>
#define INITGUID
#include <initguid.h>
#include "resource.h"

#define ResultFromShort(i)  ResultFromScode(MAKE_SCODE(SEVERITY_SUCCESS, 0, (USHORT)(i)))
extern HINSTANCE    g_hmodThisDll; /* Handle to this DLL itself */

/* Filedisk stuff */

#define DEVICE_BASE_NAME    _T("\\FileDisk")
#define DEVICE_DIR_NAME     _T("\\Device")      DEVICE_BASE_NAME
#define DEVICE_NAME_PREFIX  DEVICE_DIR_NAME     DEVICE_BASE_NAME
#define FILE_DEVICE_FILE_DISK       0x8000

#define IOCTL_FILE_DISK_OPEN_FILE   CTL_CODE(FILE_DEVICE_FILE_DISK, 0x800, METHOD_BUFFERED, FILE_READ_ACCESS | FILE_WRITE_ACCESS)
#define IOCTL_FILE_DISK_CLOSE_FILE  CTL_CODE(FILE_DEVICE_FILE_DISK, 0x801, METHOD_BUFFERED, FILE_READ_ACCESS | FILE_WRITE_ACCESS)
#define IOCTL_FILE_DISK_QUERY_FILE  CTL_CODE(FILE_DEVICE_FILE_DISK, 0x802, METHOD_BUFFERED, FILE_READ_ACCESS)

typedef struct _OPEN_FILE_INFORMATION {
    LARGE_INTEGER   FileSize;
    BOOLEAN         ReadOnly;
    USHORT          FileNameLength;
    UCHAR           FileName[MAX_PATH];
} OPEN_FILE_INFORMATION, *POPEN_FILE_INFORMATION;

void PrintLastError(PTCHAR title, DWORD flags);
void PrintLastError(PTCHAR title);

/* {844449EE-53BC-4b46-8AC4-EAF476C1CF3B} */
DEFINE_GUID(CLSID_FileDiskExtension,
0x844449ee, 0x53bc, 0x4b46, 0x8a, 0xc4, 0xea, 0xf4, 0x76, 0xc1, 0xcf, 0x3b);

#define NUMBEROFDEVICES _T("System\\CurrentControlSet\\Services\\FileDisk\\Parameters")

#define M(x) MessageBox(NULL, x, _T("FileDisk"), MB_OK);

enum
{
    TYPE_DRIVE_MOUNTED = 0,
    TYPE_DRIVE_NOTFILEDISK,
    TYPE_ISO_UNMOUNTED,
    TYPE_ISO_MOUNTED,
    TYPE_IMG_MOUNTED,
    TYPE_IMG_UNMOUNTED,
    TYPE_NOT_HANDLED
};

class CShellExtClassFactory : public IClassFactory
{
protected:
    ULONG   m_cRef; /* Object reference count */

public:
    CShellExtClassFactory();
    ~CShellExtClassFactory();

    /* IUnknown members */
    STDMETHODIMP            QueryInterface(REFIID, LPVOID FAR *);
    STDMETHODIMP_(ULONG)    AddRef();
    STDMETHODIMP_(ULONG)    Release();

    /* IClassFactory members */
    STDMETHODIMP            CreateInstance(LPUNKNOWN, REFIID, LPVOID FAR *);
    STDMETHODIMP            LockServer(BOOL);

};
typedef CShellExtClassFactory *LPCSHELLEXTCLASSFACTORY;

/* this is the actual OLE Shell context menu handler */
class CShellExt : public IContextMenu,
IShellExtInit
{
protected:
    ULONG           m_cRef; /* Object reference count */
    LPDATAOBJECT    m_pDataObj;

public:
    CShellExt();
    ~CShellExt();

    /* IUnknown members */
    STDMETHODIMP            QueryInterface(REFIID, LPVOID FAR *);
    STDMETHODIMP_(ULONG)    AddRef();
    STDMETHODIMP_(ULONG)    Release();

    /* IShell members */
    STDMETHODIMP            QueryContextMenu(HMENU hMenu, UINT indexMenu,  UINT idCmdFirst, UINT idCmdLast, UINT uFlags);
    STDMETHODIMP            InvokeCommand(LPCMINVOKECOMMANDINFO lpcmi);
    STDMETHODIMP            GetCommandString(UINT_PTR idCmd, UINT uFlags, UINT FAR *reserved, LPSTR pszName, UINT cchMax);

    /* IShellExtInit methods */
    STDMETHODIMP            Initialize(LPCITEMIDLIST pIDFolder, LPDATAOBJECT pDataObj, HKEY hKeyID);

    BOOL Mount(HWND hwnd, TCHAR DriveLetter);
    BOOL Umount(HWND hwnd);
    UINT GetType(LPTSTR szPath);
    void SetImagePath(LPTSTR szPath);

private:
    BOOL IsMounted(LPTSTR szPath);

    /* FileDisk */
    int FileDiskStatus(TCHAR DriveLetter);
    int FileDiskUmount(TCHAR DriveLetter);
    int FileDiskMount(int DeviceNumber, POPEN_FILE_INFORMATION  OpenFileInformation, TCHAR DriveLetter, BOOLEAN CdImage);

    TCHAR   m_szPath[MAX_PATH];
    TCHAR   m_szMountedImage[MAX_PATH];
    TCHAR   m_MountedOn;
    TCHAR   m_Type;
    DWORD   m_NumberOfDevices;
    HBITMAP m_hFileDiskBmp;
};

typedef CShellExt *LPCSHELLEXT;
