/*
 * FileDisk Shell Extension
 * Copyright (c) 2005-2006 Gianluigi Tiesi <sherpya@netfarm.it>
 * ShellExt Code taken from ClamWin Antivirus Scanner (c) 2004 alch,
 * FileDisk Code taken from FileDisk (c) 1999-2004 Bo Brantn.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "FileDiskExt.h"

#define BAIL(m) { M(_T(m)); return S_FALSE; }

/* Partially Ripped from 7zip :D */
#define kClsidString            _T("{844449EE-53BC-4b46-8AC4-EAF476C1CF3B}")

#define kShellExtName           _T("FileDisk Shell Extension")
#define kClsidMask              _T("CLSID\\%s")
#define kClsidInprocMask        _T("CLSID\\%s\\InprocServer32")
#define kApprovedKeyPath        _T("Software\\Microsoft\\Windows\\CurrentVersion\\Shell Extensions\\Approved")

#define kRootKeyNameForAll      _T("*\\shellex\\ContextMenuHandlers\\FileDiskExt")
#define kRootKeyNameForDrive    _T("Drive\\shellex\\ContextMenuHandlers\\FileDiskExt")

typedef struct
{
  LPCTSTR SubKey;
  LPCTSTR ValueName;
  LPCTSTR Data;
} CRegItem;

static inline bool IsItWindowsNT(void)
{
  OSVERSIONINFO versionInfo;
  versionInfo.dwOSVersionInfoSize = sizeof(versionInfo);
  if (!::GetVersionEx(&versionInfo)) return false;
  return (versionInfo.dwPlatformId == VER_PLATFORM_WIN32_NT);
}

STDAPI DllRegisterServer(void)
{
    TCHAR Filename[MAX_PATH];
    TCHAR subKey[MAX_PATH];
    HKEY hKey;
    int i = 0;
    DWORD disposition = 0;

    if (!GetModuleFileName(g_hmodThisDll, Filename, MAX_PATH))
        BAIL("Error getting dll path");

    CRegItem clsidEntries[] =
    {
        kClsidMask,         NULL,                    kShellExtName,
        kClsidInprocMask,   NULL,                    Filename,
        kClsidInprocMask,   _T("ThreadingModel"),    _T("Apartment"),
        NULL,               NULL,                    NULL
    };

    for(i = 0; clsidEntries[i].Data; i++)
    {
        _stprintf(subKey, clsidEntries[i].SubKey, kClsidString);

        if (RegCreateKeyEx(HKEY_CLASSES_ROOT, subKey, NULL, 0, REG_OPTION_NON_VOLATILE,
            KEY_WRITE, NULL, &hKey, &disposition) != ERROR_SUCCESS)
            BAIL("Error opening HKCR Registry Key");

        if (RegSetValueEx(hKey, clsidEntries[i].ValueName, 0, REG_SZ, (BYTE *) clsidEntries[i].Data,
            (DWORD) _tcslen(clsidEntries[i].Data) * sizeof(TCHAR)) != ERROR_SUCCESS)
        {
            RegCloseKey(hKey);
            BAIL("Error creating Shell Extension Registry Key");
        }
        RegCloseKey(hKey);
    }

    /* kRootKeyNameForAll "*" */
    if (RegCreateKeyEx(HKEY_CLASSES_ROOT, kRootKeyNameForAll, NULL, 0, REG_OPTION_NON_VOLATILE,
            KEY_WRITE, NULL, &hKey, &disposition) != ERROR_SUCCESS)
            BAIL("Error opening HKCR * Key");

    if (RegSetValueEx(hKey, NULL, 0, REG_SZ, (BYTE *) kClsidString,
        (DWORD) _tcslen(kClsidString) * sizeof(TCHAR)) != ERROR_SUCCESS)
    {
        RegCloseKey(hKey);
        BAIL("Error creating * Menu Handler Registry Key");
    }
    RegCloseKey(hKey);

    /* kRootKeyNameForDrive "Drive" */
    if (RegCreateKeyEx(HKEY_CLASSES_ROOT, kRootKeyNameForDrive, NULL, 0, REG_OPTION_NON_VOLATILE,
            KEY_WRITE, NULL, &hKey, &disposition) != ERROR_SUCCESS)
            BAIL("Error opening HKCR * Key");

    if (RegSetValueEx(hKey, NULL, 0, REG_SZ, (BYTE *) kClsidString,
        (DWORD) _tcslen(kClsidString) * sizeof(TCHAR)) != ERROR_SUCCESS)
    {
        RegCloseKey(hKey);
        BAIL("Error creating * Menu Handler Registry Key");
    }
    RegCloseKey(hKey);

    if(IsItWindowsNT())
    {
        if (RegCreateKeyEx(HKEY_LOCAL_MACHINE, kApprovedKeyPath, NULL, 0, REG_OPTION_NON_VOLATILE,
            KEY_WRITE, NULL, &hKey, &disposition) != ERROR_SUCCESS)
            BAIL("Error opening Approved Extensions Registry Key");


        if (RegSetValueEx(hKey, kClsidString, 0, REG_SZ, (BYTE *) kShellExtName,
            (DWORD) _tcslen(kShellExtName) * sizeof(TCHAR)) != ERROR_SUCCESS)
        {
            RegCloseKey(hKey);
            BAIL("Error creating Shell Ext Registry Key");
        }
        RegCloseKey(hKey);
    }

    return S_OK;
}

STDAPI DllUnregisterServer(void)
{
    TCHAR subKey[MAX_PATH];

    _stprintf(subKey, kClsidInprocMask, kClsidString);
    RegDeleteKey(HKEY_CLASSES_ROOT, subKey);

    _stprintf(subKey, kClsidMask, kClsidString);
    RegDeleteKey(HKEY_CLASSES_ROOT, subKey);

    RegDeleteKey(HKEY_CLASSES_ROOT, kRootKeyNameForAll);
    RegDeleteKey(HKEY_CLASSES_ROOT, kRootKeyNameForDrive);

    if(IsItWindowsNT())
    {
        HKEY hKey;
        if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, kApprovedKeyPath, 0, KEY_SET_VALUE, &hKey) == NOERROR)
        {
            RegDeleteValue(hKey, kClsidString);
            RegCloseKey(hKey);
        }
    }
    return ERROR_SUCCESS;
}
