/*
 * FileDisk Shell Extension
 * Copyright (c) 2005-2007 Gianluigi Tiesi <sherpya@netfarm.it>
 * ShellExt Code taken from ClamWin Antivirus Scanner (c) 2004 alch,
 * FileDisk Code taken from FileDisk (c) 1999-2004 Bo Brantn.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "FileDiskExt.h"

HWND MainDlg = NULL;
TCHAR szImageName[MAX_PATH];
LPCSHELLEXT pShellExt = NULL;

void FileDiskMountImage(HWND hwnd)
{
    LRESULT res = 0;

    res = SendMessage(GetDlgItem(MainDlg, IDC_COMBO), CB_GETCURSEL, 0, 0);
    if (res == CB_ERR) goto error_image;

    res = SendMessage(GetDlgItem(MainDlg, IDC_COMBO), CB_GETITEMDATA, res, 0);
    if (res == CB_ERR) goto error_image;

    if (!pShellExt->Mount(hwnd, (TCHAR) res + 65))
        PrintLastError(_T("FileDisk MountImage"), MB_ICONERROR);

    return;

error_image:
    PrintLastError(_T("FileDisk MountImage"), MB_ICONERROR);
}

INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch(uMsg)
    {
        case WM_CLOSE:
            DestroyWindow(hwndDlg);
            return true;
        case WM_DESTROY:
            PostQuitMessage (0);       /* send a WM_QUIT to the message queue */
            return true;
        case WM_COMMAND:
            switch (wParam)
            {
                case IDC_BUTTON_MOUNT:
                    FileDiskMountImage(hwndDlg);
                case IDC_BUTTON_CANCEL:
                    DestroyWindow(hwndDlg);
                    break;
            }
            return true;
    }
    return false;
}

/* Exported */
HRESULT MountImage(HWND hwnd, HINSTANCE inst, LPSTR cmdline_in, INT cmdshow)
{
    MSG messages;
    int idx = 0;
    int i = 0;
    DWORD res = 0;
    TCHAR szItem[32];
    bool once = true;

#ifdef UNICODE
    WCHAR cmdline[MAX_PATH];
    cmdline[0] = L'\0';
    MultiByteToWideChar(CP_ACP, MB_ERR_INVALID_CHARS, cmdline_in, -1, cmdline, MAX_PATH);
#else
    CHAR cmdline[MAX_PATH];
    cmdline[0] = '\0';
    strncat(cmdline, cmdline_in, MAX_PATH);
#endif

    if (!cmdline[0])
    {
        MessageBox(hwnd, _T("Invalid Image Specified"), _T("FileDisk"), MB_OK | MB_ICONERROR);
        return FALSE;
    }

    memset(&messages, 0, sizeof(messages));

    pShellExt = new CShellExt();

    pShellExt->SetImagePath(cmdline);

    switch (pShellExt->GetType(cmdline))
    {
        case TYPE_ISO_UNMOUNTED:
        case TYPE_IMG_UNMOUNTED:
            break;
        case TYPE_ISO_MOUNTED:
        case TYPE_IMG_MOUNTED:
            MessageBox(hwnd, _T("The selected image is already mounted"), _T("FileDisk"), MB_OK);
            goto end_mountimage;
        default:
            MessageBox(hwnd, _T("Invalid Image Specified"), _T("FileDisk"), MB_OK | MB_ICONERROR);
            goto end_mountimage;
    }

    MainDlg = CreateDialog(g_hmodThisDll, MAKEINTRESOURCE(IDD_MOUNT_IMAGE), NULL, DialogProc);

    if (!MainDlg)
    {
        PrintLastError(_T("FileDisk OpenIso"), MB_ICONERROR);
        return FALSE;
    }

    /* Set Image Name */
    SetWindowText(GetDlgItem(MainDlg, IDC_IMAGE_NAME), cmdline);

    /* Fill combo box with drives entries */

    res = GetLogicalDrives();

    SendMessage(GetDlgItem(MainDlg, IDC_COMBO), CB_RESETCONTENT, 0, 0);

    for (i = 0; i < 26; i++)
    {
        if ((res >> i) & 0x1)
            continue;

        _stprintf(szItem, _T(" On letter %c:"), 65 + i);
        SendMessage(GetDlgItem(MainDlg, IDC_COMBO), CB_ADDSTRING, 0, (LPARAM) szItem);
        SendMessage(GetDlgItem(MainDlg, IDC_COMBO), CB_SETITEMDATA, idx, (LPARAM) i);
        idx++;

        /* Select the first item */
        if (once)
        {
            SendMessage(GetDlgItem(MainDlg, IDC_COMBO), CB_SELECTSTRING, 0, (LPARAM) szItem);
            once = false;
        }
    }

    ShowWindow(MainDlg, SW_SHOWNORMAL);

    while (GetMessage (&messages, NULL, 0, 0))
    {
        TranslateMessage(&messages);
        DispatchMessage(&messages);
    }

end_mountimage:
    if (pShellExt) delete pShellExt;
    return (HRESULT) messages.wParam;
}
